@@ -217,12 +217,16 @@
 		pthread_mutex_trylock(&(STREAM)->lock); \
 	}
 
+#define __STDIO_SET_USER_LOCKING(STREAM)   	((STREAM)->user_locking = 1)
+
 #else  /* __STDIO_THREADSAFE */
 
 #define __STDIO_THREADLOCK(STREAM)
 #define __STDIO_THREADUNLOCK(STREAM)
 #define __STDIO_THREADTRYLOCK(STREAM)
 
+#define __STDIO_SET_USER_LOCKING(STREAM)
+
 #endif /* __STDIO_THREADSAFE */
 
 /* This file may eventually have two personalities:
@@ -435,7 +439,7 @@ extern int _cs_close(void *cookie);
 							? (*(stream)->bufpos++) = (c) \
 							: fputc_unlocked((c),(stream)) )
 #else  /* __STDIO_PUTC_MACRO */
-#define __PUTC(c, stream)	fputc_unlocked(c, stream);
+#define __PUTC(c, stream)	fputc_unlocked(c, stream)
 #endif /* __STDIO_PUTC_MACRO */
 
 
